# How to use the template

## Summmary

This template is intended for use by [Moonshine-IDE](https://github.com/Moonshine-IDE/Moonshine-IDE).

Goals:
- Generate a Java Domino project with CRUD agents for each form, based on the templates in this project
- Support easy template updates as development continues on Moonshine-Domino-CRUD
- Support project and file strucuture changes without requiring updates in the code.  New insertion parameters or field properties/categories will still potentially require configuration updates.


## Organization

There are two primary directories:
- `project`:  Defines the project structure and file templates
- `elements`:  templates for field-related code snippets within the files


## Usage

This is the proposed algorithm for generating projects from the below templates.

You can see an example Groovy implementation of this template 
[here](https://github.com/Moonshine-IDE/Moonshine-Domino-CRUD/blob/main/src/test/groovy/helper/GenerateTemplate.groovy).

I am using the format `%parametername%` for the insertion parameters in both file names and file contents.
The only problems I have had with this format so far is when using the [Copy task] in `build.gradle` (a directory named with only an insertion parameter is ignored).
If we find more problems (with Flex, Haxe, Windows, etc.) we can consider changing the format.

If the project is generated properly, there should be no remaining insertion parameters.  
I have a [test](https://github.com/Moonshine-IDE/Moonshine-Domino-CRUD/blob/main/src/test/groovy/template/TestTemplateSpec.groovy) 
for this as part of the `Moonshine-Domino-CRUD` test suite.


### Project-Level Population

For each `.template` file in `template/project`, create a matching file without the `.template` extension.  
The following insertion parameters should be applied to the file names and contents:

Parameter | Description
----------|------------
`%project%` | The name of the project
`%NotesExecutablePath%` | The Notes path from the Domino settings
`%eachform%` | See special logic below


### `%eachform%`

For each directory containing `%eachform%`, generate a corresponding directory for each converted form.  `%eachform%` must be the only insertion parameter in this case.

Files should be created in each directory for each file inside the `%eachform%` directory.  The following insertion parameters should be applied to the file names and contents for each `.template` file.

Parameter | Description
----------|------------
`%form%` | The Form Name, but normalized for use in code.  All characters besides letters, numbers, and `_` should be removed.
`%formRaw%` | The original Form Name
`%view%`| The specified view name.

Also include:
- The values generated in [Element Population](#element-population) as insertion parameters
- The parameters from [Project-Level Population](#project-level-population)


## Element Population

The `.template` files in `templates/elements` are used to generate additional field-specific insertion parameters that will be used in the [%eachform%] step above.  
The file path relative to `templates/elements` (excluding `.template` determines the insertion parameter names.  
So, `templates/elements/all/FieldList.template` should generate a `%all/FieldList%` insertion parameter.

There are subdirectories in `elements` that determine the categories of fields that are applicable for the given entry.  
Fields may belong to more than one category.  See the definitions below.

Category | Definition
---------|-----------
all | All fields
key | Field is included in view, and Sorted option is anything except "No sorting".  The order from the field list must be maintained.  This will be used for document lookups.  Notes:  for convenience, only one key is recommended for most cases.  DateType fields are discouraged.  RichText fields are not allowed.  A "Compute on compose" field could be used to populate a value automatically and restrict further updates.
editable | Editable selection is "Editable". Determines inclusion in Create and Update agents
required | TODO:  There is no option for this in FormBuilder or DominoVisualEditor currently, so all fields will be treated as not required.   This may be added later, and it would be used to trigger validation for certain parameters.

For each `.template` file, iterate over the applicable fields and apply the insertion parameters below.  

Parameter | Description
----------|------------
`%name%` | The name of the field
`%label%` | The label for the field. Only used for comments
`%description%` | The description of the field.  Only used for comments
`%type%` | The type of the field:  `text`, `number`, `datetime`, or `richtext` 
`%typeupper%` | The type of the field, in uppercase:  `TEXT`, `NUMBER`, `DATETIME`, or `RICHTEXT`
`%multivalue%` | Boolean indicating whether it is a multivalue field.  `true` or `false`
TODO:  other properties?  They are not relevant for the current code
TODO:  match Moonshine format when possible

The results should then be concatenated.  Do not add any additional separators - this should be included in the `.template` files already.
